@@ -3,6 +3,9 @@ import sys
 #import types
 PY2 = sys.version_info[0] == 2
 
+import os,datetime,logging
+l = logging.getLogger('ppssauth')
+
 if PY2:
     def is_nonstr_iter(v):
         return hasattr(v, '__iter__')
@@ -32,12 +35,61 @@ class Conf():
         return path + "/" + filename + "." + cls.templateextension
 
     @staticmethod
-    def conf2list(confval):
-        if not is_nonstr_iter(confval):
-            #print(repr(confval))
-            confval = aslist(confval,flatten=False)
-        print(repr(confval))
-        return confval
+    def conf2list(confval,default=None):
+        try:
+            if not is_nonstr_iter(confval):
+                #print(repr(confval))
+                confval = aslist(confval,flatten=False)
+            print(repr(confval))
+            return confval
+        except Exception as e:
+            l.exception(u"Exception parsing value:{}".format(confval))
+            if default is None:
+                raise e
+            else:
+                return default
+
+    @staticmethod
+    def parseSmtp(val):
+        """ 
+            try to parse smtp uri in a (user,password,host,port) tuple
+            Full format:
+            user:password@host:port
+            optionals:
+            [user[:password]@]host[:port]
+        """
+        if val is None:
+            return None,None,None,None
+        try:
+            
+            user = None
+            password = None
+            host = None
+            port = 25
+            at = s.rfind('@')
+            if at>0:
+                hostpart = val[at:]
+                userpart = val[:at]
+            else:
+                hostpart = val
+                userpart = None
+            if userpart:
+                colonindex = userpart.find(":")
+                if colonindex>=0:
+                    user = userpart[:colonindex]
+                    password = userpart[colonindex:]
+                else:
+                    user = userpart
+            colonindex = hostpart.find(":")
+            if colonindex>=0:
+                host,post = hostpart.split(":")
+            else:
+                host = hostpart
+            return user,password,host,port
+        except Exception as e:
+            l.exception("error parsing smtp value '{}'".formaT(val))
+            return None,None,None,None
+
 
     @classmethod
     def setup(cls, settings):
@@ -48,14 +100,18 @@ class Conf():
         # routes
         cls.loginurl = settings.get("ppss_auth.login_url", "/login")
         cls.logouturl = settings.get("ppss_auth.logout_url", "/logout")
-        cls.registeruurl = settings.get("ppss_auth.register_url", "/register")
-
         cls.postlogoutroute = settings.get("ppss_auth.post_logout_route", "ppsslogin")
         cls.postloginroute = settings.get("ppss_auth.post_login_route", "home")
         cls.postloginfollow = settings.get("ppss_auth.post_login_follow", "true").lower() == 'true'
 
         cls.backfromppssauth = settings.get("ppss_auth.back_from_ppss_auth",None)
         
+        # NEW USER REGISTRATION
+        cls.registeruurl = settings.get("ppss_auth.register_url", "/register")
+        cls.usercanregister = settings.get("ppss_auth.canregister", "false").lower() == "true"
+        cls.activationemail = settings.get("ppss_auth.register_requireactivation", "true").lower() != "false"
+        cls.activationemailtpl = settings.get("ppss_auth.register_activationemailtpl","")
+
         cls.saltforhash = settings.get("ppss_auth.salt", "ImTheSaltOfThisLife")
 
         cls.forbiddentologin = settings.get("ppss_auth.forbidden_to_login", "true").lower() == 'true'
@@ -120,3 +176,9 @@ class Conf():
 
         cls.idclient = "896890850990-kdppue1lijfhkj5s7tu8lsh7gsj99ndj.apps.googleusercontent.com"
         cls.secret = "hhuJLuYo9ZnvAVd9XopH-s8n"
+
+
+        #SMTP
+        cls.smtpuri = settings.get("ppss_auth.smpt",None)
+        cls.smtpuser,cls.smtppassword,cls.smtphost,cls.smtport = Conf.parseSmtp(cls.smtpuri)
+        cls.mailsender = settings.get("ppss_auth.mailsender",None)
