@@ -12,6 +12,7 @@ from ..ppss_auth_utils import checkPassword
 from pyramid.view import view_config,forbidden_view_config
 from pyramid.httpexceptions import HTTPFound, HTTPNotFound
 from beaker.cache import cache_region
+from sqlalchemy.exc import InvalidRequestError
 
 
 
@@ -42,7 +43,7 @@ class ACLBuilder(object):
 
     @cache_region('short_term', 'ACLpermissions')
     def buildACL(self):
-        
+        l.debug("buildACL cache miss")
         acl = [] 
         try:
             groups = self.session.query(PPSsgroup).filter(PPSsgroup.enabled==1).all()
@@ -51,17 +52,21 @@ class ACLBuilder(object):
                         str("g:"+group.name),
                         tuple([str(p.name) for p in group.permissions])  
                 ) )
+        except InvalidRequestError as invalid_session:
+            l.exception("**************invalid_session. Something is going very very wrong!!")
+            #raise invalid_session
         except Exception as e:
-            l.exception("called without a transaction")
+            l.warn("called without a transaction")
         acl = self.baseACL + acl
-        l.warn("ACLBuilder:acl built: {acl}".format(acl = acl))
+        l.info("ACLBuilder:acl built: {acl}".format(acl = acl))
         return acl
 
 
 #This class stores the acl structure used by the root factory
 class ACLRoot(object):
     baseACL=[(Allow, Authenticated, 'view'),
-        (Allow, 'g:'+constants.SYSADMINGROUP, ALL_PERMISSIONS)
+        (Allow, 'g:'+constants.SYSADMINGROUP, ALL_PERMISSIONS),
+        (Allow, 'ppss_auth:changepassword',"ppss_auth_changepassword")
         ]
 
     lastupdateACL = datetime.datetime.now()
@@ -179,6 +184,8 @@ class AuthController():
 
                 r.session[Conf.sessionuser] = {'id':res.id,'name':username,'user':res}
                 headers = remember(r, res.id)
+                if res.passwordExpired(): 
+                    return HTTPFound(r.route_url("ppss:user:changepassword"))
                 return HTTPFound(postloginpage,headers=headers)
             self.request.dbsession.query(PPSsuser).filter()
             l.warn("Login attempt failed for user {user}".format(user=username))    
@@ -203,6 +210,8 @@ class AuthController():
         return Response("OK")
 
     def registernewuser(self):
+        if not Conf.usercanregister:
+            HTTPFound(self.request.HTTPFound('ppsslogin'))
         retdict = self.retdict
         retdict["link"] = ["",""]
         if self.request.POST:
@@ -232,12 +241,11 @@ class AuthController():
                 self.request.dbsession.add(user)
                 retdict['msg'] = "User created, please go to login page."
                 retdict["link"] = ["login",self.request.route_url("ppsslogin")]
-
             
             HTTPFound(self.request.route_url("ppsslogin"))
         return retdict
         
-    @view_config(route_name="ppss:user:changepassword",renderer=Conf.changepasswordtemplate,permission='login')
+    @view_config(route_name="ppss:user:changepassword",renderer=Conf.changepasswordtemplate,permission='ppss_auth_changepassword')
     def ppsschangepassword(self):
         l.debug("change password")
         if not Conf.sessionuser in self.request.session:
@@ -255,10 +263,10 @@ class AuthController():
             res = PPSsuser.checkLogin(username,oldpassword,self.request.dbsession)     
             if res:
                 res.password = newpassword       
-                
+                #TODO: execute fake login with post login redirect?
             else:
                 retdict['res']=False
-                retdict['msg']='password is wrong'
+                retdict['msg']='Old password is wrong'
         return retdict
         
     def listUser(self):
@@ -280,10 +288,20 @@ class AuthController():
         else:
             if userid == self.request.loggeduser.id:
                 selfediting = True
-            submiturl = self.request.route_url('ppss:user:edit',elementid=userid)
+                submiturl = self.request.route_url('ppss:user:editself')
+            else:
+                submiturl = self.request.route_url('ppss:user:edit',elementid=userid)
 
         l.debug("***{id} -> {user}".format(user=user,id=userid))
-        retdict = dict(self.retdict,**{'msg':"",'res':True,'userid':userid,'submiturl':submiturl} )
+        retdict = dict(self.retdict,
+            **{
+                'msg':"",
+                'res':True,
+                'userid':userid,
+                'submiturl':submiturl,
+                'selfediting':selfediting
+        })
+
         if not user:
             retdict['res'] = False
             retdict['msg'] = "user not found"
@@ -299,11 +317,15 @@ class AuthController():
             editablegroups = logged_user.groups
         l.debug("editablegroups = {}".format(editablegroups))
 
-        retdict.update({"user" : user, 'allgroups':editablegroups })
+
+        requireoldpassowrd = userid >= 0 and ( selfediting or logged_user.isSuperUser()==False )
+
+        retdict.update({"user" : user, 'allgroups':editablegroups,'requireoldpassowrd':requireoldpassowrd })
 
         if self.request.POST:
             canenable = True
             if userid<0:
+                l.debug("this is a post for creation")
                 username = self.request.params.get("username",None)
                 if not username:
                     retdict['msg'] = "Username can not be empty."
@@ -319,18 +341,21 @@ class AuthController():
 
             if not user:
                 return retdict
+            
             newpassword = self.request.params.get("password","")
+            ##change user password
             if newpassword:
-                
                 confirmnewpassword = self.request.params.get("confirmnewpassword","")
                 currentpassword = self.request.params.get("currentpassword","")
-
+                l.debug("new password match? {}".format(newpassword==confirmnewpassword) )
                 #existing user must match older password (unless edited by superadmin)
-                if (self.request.loggeduser.isSuperUser() == False) or selfediting:
+                #if (self.request.loggeduser.isSuperUser() == False) or selfediting:
+                if requireoldpassowrd:
                     if userid>=0 and (not PPSsuser.checkLogin(username,currentpassword,self.request.dbsession)): 
-                        retdict['msg'] = "your current password does not match" 
-
-                elif newpassword==confirmnewpassword:
+                        retdict['msg'] = "{}urrent password does not match!".format("Your c" if selfediting else "C")
+                        return retdict
+                        
+                if newpassword==confirmnewpassword:
                     l.info("*****changing password for {}".format(user.username) )
                     res = checkPassword(user,newpassword)
                     l.info("checkPassword result for user {} = {}".format(user.username, res ))
@@ -354,19 +379,19 @@ class AuthController():
 
         return retdict
 
-    def checkPassword(self):
-        newpassword = self.request.get("newpassword","")
-        if self.request.session[Conf.sessionuser]['id'] != self.userid and not self.request.session['admin']: 
-            return {"res":False,"message":"Utente non autorizzato"}
-            #TODO: log the incident
-        if not self.user:
-            return {"res":False,"message":"Utente non trovato"}
+    # def checkPassword(self):
+    #     newpassword = self.request.get("newpassword","")
+    #     if self.request.session[Conf.sessionuser]['id'] != self.userid and not self.request.session['admin']: 
+    #         return {"res":False,"message":"Utente non autorizzato"}
+    #         #TODO: log the incident
+    #     if not self.user:
+    #         return {"res":False,"message":"Utente non trovato"}
         
-        res = checkPassword(self.user,newpassword)
-        if res: 
-            return {"res":res,"message":"ok"}
-        else:
-            return {"res":res,"message":Conf.passwordwrongmessage}
+    #     res = checkPassword(self.user,newpassword)
+    #     if res: 
+    #         return {"res":res,"message":"ok"}
+    #     else:
+    #         return {"res":res,"message":Conf.passwordwrongmessage}
 
 
     def listGroup(self):
